import os
import json
import os
import hashlib
import sys
import time
import requests
from zvolv_sdk.server_configuration import EnvVariables

class Client():

    folder_path = "/tmp"            # For server
    # folder_path = os.getcwd()     # For local testing
    CACHE_FILE_NAME = 'token_cache.json'
    CACHE_FILE_PATH = os.path.join(folder_path, CACHE_FILE_NAME)

    def __init__(
        self,
        zvolv_user_id: str = EnvVariables.get_zvolv_user_id(),
        zvolv_password: str = EnvVariables.get_zvolv_password(),
        zvolv_domain: str = EnvVariables.get_zvolv_domain(),
        zvolv_business_tag_id: str = "98NCMBD2KBZ4R",
        zvolv_service_url: str = EnvVariables.get_zvolv_localhost_url()
        
    ) -> None:
        self.zvolv_user_id = zvolv_user_id
        self.zvolv_password = zvolv_password
        self.zvolv_domain = zvolv_domain
        self.zvolv_business_tag_id = zvolv_business_tag_id
        self.zvolv_service_url = zvolv_service_url

    
    def __repr__(self):
        return '{}'.format(
            repr(self.login()),
        )

    def load_token_cache(self):
        """ Checking whether file is present or not, Creating file with value {} and  Reading value from file """
        if not os.path.isfile(self.CACHE_FILE_PATH):
            with open(self.CACHE_FILE_PATH, 'w') as file:
                json.dump({}, file)

        with open(self.CACHE_FILE_PATH, 'r') as cache_file:
            return json.load(cache_file)

    def save_token_cache(self,token_cache):
        with open(self.CACHE_FILE_PATH, 'w') as cache_file:
            json.dump(token_cache, cache_file)
    

    def login(self):
            token_cache = self.load_token_cache()
            business_domain = self.zvolv_domain
            if token_cache and business_domain in token_cache and token_cache and (time.time() - token_cache[business_domain]["serverTime"]) < 60:
                print("token cache")
                return token_cache[business_domain]

            sha512pwd = hashlib.sha512(self.zvolv_password.encode('utf-8')).hexdigest()
            payload = {'email': self.zvolv_user_id, 'password': sha512pwd}
            headers = {'businessDomain': self.zvolv_domain, 'jwt': 'true', 'businessTagID': self.zvolv_business_tag_id ,'Device':'browser'}
            login_url = f"{self.zvolv_service_url}{EnvVariables.get_api_17_version()}user/login"
            login_response = requests.post(url=login_url, data=payload, headers=headers)

            if login_response.status_code == 200:
                token_cache = {
                    business_domain: {}
                }
                login_response = login_response.json()
                for key, value in login_response.items():
                    token_cache[business_domain][key] = value
                self.save_token_cache(token_cache)
                return login_response
            else:
                raise Exception()


import pytest
import requests_mock
from zvolv_sdk.client import Client

@pytest.fixture
def mock_requests_post_success(requests_mock):

    mock_url = 'https://yogeshjadhav.zvolv.co/rest/v17/user/login'
    mock_response = {'token': 'mock_token'}
    requests_mock.post(mock_url, json=mock_response)
    return mock_response

@pytest.fixture
def mock_requests_post_failure(requests_mock):
    mock_url = 'https://yogeshjadhav.zvolv.co/rest/v17/user/login'
    requests_mock.post(mock_url, status_code=401)  # Simulating a failed login
    return {'error': 'Authentication failed'}

def test_login_successful(mock_requests_post_success):
    instance = Client()  # Create an instance of your class
    result = instance.login()
    assert result == mock_requests_post_success

def test_login_failure(mock_requests_post_failure):
    instance = Client()  # Create an instance of your class
    with pytest.raises(Exception):
        instance.login()



# @pytest.fixture
# def mock_requests_post(requests_mock):
#     # Arrange
#     mock_url = 'https://yogeshjadhav.zvolv.co/rest/v17/user/login'
#     mock_response = {'token': 'mock_token'}
#     requests_mock.post(mock_url, json=mock_response)
#     return mock_response

# def test_login_successful(mock_requests_post):
#     instance = Client()  # Create an instance of your class
#     result = instance.login()
#     assert result == mock_requests_post

# def test_login_successful():
#     mock_url = 'https://yogeshjadhav.zvolv.co/rest/v17/user/login'
#     with requests_mock.Mocker() as m:
#         m.post(mock_url, status_code=200, json={'token': 'mock_token'})

#         instance = Client()
#         result = instance.login()
#         assert result == {'token': 'mock_token'}



import pytest
from unittest.mock import patch, MagicMock
from zvolv_sdk.client_one import Client_one  # Import the class containing the login method

@pytest.fixture
def mock_load_token_cache(requests_mock):
    # Arrange
    return MagicMock(return_value={})  # Mock the load_token_cache method

@pytest.fixture
def mock_save_token_cache(requests_mock):
    # Arrange
    return MagicMock()  # Mock the save_token_cache method

@pytest.fixture
def mock_requests_post_success(requests_mock):
    # Arrange
    mock_url = 'https://yogeshjadhav.zvolv.co/rest/v17/user/login'
    mock_response = {'token': 'mock_token'}
    requests_mock.post(mock_url, json=mock_response)
    return mock_response

@pytest.fixture
def instance(mock_load_token_cache, mock_save_token_cache):
    # Arrange
    with patch.object(Client_one, 'load_token_cache', mock_load_token_cache):
        with patch.object(Client_one, 'save_token_cache', mock_save_token_cache):
            instance = Client_one()  # Create an instance of your class
    return instance

def test_login_successful(instance, mock_requests_post_success):
    # Arrange
    instance.load_token_cache.side_effect = [{}]

    # Act
    result = instance.login()

    # Assert
    assert result == mock_requests_post_success
    instance.save_token_cache.assert_called_once()  # Assert that save_token_cache is called

# Add more test cases for different scenarios, such as cache hit, failure, etc.



import os
from dotenv import load_dotenv

class EnvVariables(object):
    if(os.path.exists("../../.env")):
        load_dotenv()

    # @staticmethod
    # def get_val(key):
    #     return os.environ[key] if key in os.environ else None

    @staticmethod
    def get_val(key):
        try:
            return os.environ[key]
        except KeyError:
            raise Exception(f"error {key} variable not found in env")

    @staticmethod
    def get_zvolv_localhost_url():
        return EnvVariables.get_val('LOCALHOST_URL')

    @staticmethod
    def get_api_base_url():
        return EnvVariables.get_val('BASE_URL')

    @staticmethod
    def get_api_version():
        return EnvVariables.get_val('API_VERSION')

    @staticmethod
    def get_api_17_version():
        return EnvVariables.get_val('API_VERSION_17')

    @staticmethod
    def get_zvolv_user_id():
        return EnvVariables.get_val('ADMIN_EMAIL')

    @staticmethod
    def get_zvolv_password():
        return EnvVariables.get_val('ADMIN_PASSWORD')

    @staticmethod
    def get_business_tag_id():
        return EnvVariables.get_val('BUSINESS_TAG_ID')

    @staticmethod
    def get_zvolv_domain():
        return EnvVariables.get_val('BUSINESS_DOMAIN')